package task

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"git.listensoft.net/tool/jspkit/common"
	"git.listensoft.net/tool/jspkit/common/lxhttp"
	"git.listensoft.net/tool/jspkit/common/models"
	"git.listensoft.net/tool/jspkit/common/variable"
	"git.listensoft.net/tool/jspkit/logger"
	"git.listensoft.net/tool/jspkit/taxerr"
	"github.com/spf13/viper"
	"github.com/tidwall/gjson"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

const taxMaxTime = 20 * time.Minute //超时默认20分钟

// GetContent 根据地区定制Content 设置超时时间
func GetContent(childCtx context.Context, address string, pdfTel string, ticker *time.Ticker) (childCtx1 context.Context) {
	if address == "shandong" || address == "neimenggu" || address == "jiangsu" { // 针对20分钟不够用的地区
		childCtx1, _ = context.WithTimeout(childCtx, 30*time.Minute)
	} else if address == "sichuan" || address == "chongqing" || address == "shanxi" || (address == "anhui") {
		childCtx1, _ = context.WithTimeout(childCtx, 40*time.Minute)
	} else if address == "fujian" || address == "beijing" || address == "qinghai" || address == "xiamen" {
		childCtx1, _ = context.WithTimeout(childCtx, 40*time.Minute)
	} else {
		childCtx1, _ = context.WithTimeout(childCtx, taxMaxTime)
	}
	if pdfTel != "" {
		//childCtx1, _ = context.WithTimeout(childCtx, 120*time.Minute) //四川的pdf采集 一个小时不够
		ticker = time.NewTicker(4 * time.Minute) // 每四分钟加五分钟的锁
		go func() {
			for t := range ticker.C { // 阻塞等待接收ticker发出的时间信号
				common.AddTelLockerX2(pdfTel, "300")
				logger.Info("Tick at:", t)
			}
		}()
	}
	return
}

// GetTaxTask 获取任务
func GetTaxTask(robotName string) (list []models.TaxTask, err error) {
	client := lxhttp.NewHttpClient()
	defer client.CloseIdleConnections()
	param := map[string]string{
		"robotName": robotName,
	}
	bys, err2 := lxhttp.POSTJson(client, variable.TaxTaskURL+"/api/v1/getRobotQueue", param, map[string]string{})
	if err2 != nil {
		return
	}
	data := gjson.GetBytes(bys, "data").String()
	list = []models.TaxTask{}
	err = json.Unmarshal([]byte(data), &list)
	return
}

// EndQueue 释放机器人
func EndQueue(robotName string) {
	client := lxhttp.NewHttpClient()
	defer client.CloseIdleConnections()
	param := map[string]string{
		"robotName": robotName,
	}
	bys, err2 := lxhttp.POSTJson(client, variable.TaxTaskURL+"/api/v1/endQueue", param, map[string]string{}) // https://task.listensoft.net
	if err2 != nil {
		return
	}
	logger.Info("endQueue - ", string(bys))
}

// GetRobotName 获取当前机器人名称
func GetRobotName() string {
	//环境变量中读取名称
	envRobotName := os.Getenv("robotName")
	logger.Info("envRobotName:", envRobotName)
	if envRobotName != "" {
		return envRobotName
	}
	//config中读取文件
	confRobotName := viper.GetString("robotName")
	logger.Info("confRobotName:", confRobotName)
	if confRobotName != "" { //如果是测试环境 固定一个 robotName 方便调试 一直获取一个任务
		return confRobotName
	}
	//读取不到默认文件夹名称
	dir, _ := os.Getwd()
	_, file := filepath.Split(dir)
	logger.Info("fileDIrRobotName:", file)
	return file
}

// 结束任务
func EndTask(ctx context.Context, tasks []models.TaxTask) {
	clientURL := variable.TaxTaskURL + "/api/v1/endTaskTax"
	for k, v := range tasks {
		symbolToRemove := ","
		// 使用Trim函数删除字符串两端的指定符号
		tasks[k].Result.BusinessImg = strings.Trim(v.Result.BusinessImg, symbolToRemove)
	}
	for _, task := range tasks {
		// 检查 如果截图lv《申报清册_有遗漏》 状态不是有遗漏 这边强制改成有遗漏 遇到别的情况再额外处理
		if task.TaskName == variable.TaxJcShenBao &&
			(task.Result.BusinessStatus != variable.TaxSuccessdOmit && task.Result.BusinessStatus != variable.TaxFail) &&
			strings.Contains(task.Result.BusinessImg, "申报清册_有遗漏") {
			task.Result.BusinessStatus = variable.TaxSuccessdOmit
			task.Result.BusinessLog = "申报清册有遗漏,请看截图" + task.Result.BusinessLog
		}
		if task.ReqNo == "" {
			logger.Info("跳过用于解锁的空任务：", task)
			continue
		}
		task.Result.ReqNo = task.ReqNo //reqNo 返回
		if task.Result.Status == 0 {   //如果没有状态 申报检查会带状态过来
			task.Result.Status = variable.TaxSuccess //默认任务成功
		}
		if !task.Result.BusinessStatus.Check() { //没有状态的话
			task.Result.BusinessStatus = variable.TaxFail                                             //失败
			task.Result.BusinessLog = common.HandleError(ctx, taxerr.NewWebStuckTitle(false)).Error() //提示不支持的任务
		}
		//if task.Result.BusinessStatus.Success() { //如果是成功的话
		//	task.Result.BusinessLog = "成功" //提示不支持的任务
		//}
		//if task.TaskName == variable.TaxCjInInvoice && strings.Contains(task.Result.BusinessLog, "请勾选认证后再次发起采集") {
		//	task.Result.BusinessStatus = variable.TaxSuccessNeedPay
		//}
		if strings.Contains(string(task.TaskName), `kk`) && task.Result.BusinessStatus != variable.TaxTiJiaoJinSan {
			if task.Result.BusinessStatus != variable.TaxSuccessPaid && task.Result.BusinessStatus != variable.TaxSuccessdPaid {
				task.Result.BusinessStatus = variable.TaxFail
			} else {
				if task.TaskName != variable.TaxKkShebao { //社保的扣款任务要保留原状态作为 系统扣款 和 非系统扣款
					task.Result.BusinessStatus = variable.TaxSuccess
				}
			}
		}
		bys, _ := json.Marshal(task.Result)
		logger.Info("结束任务：", string(bys))
		for i := 0; i < 6; i++ {
			logger.Info(time.Now(), fmt.Sprintf("-----------第%s次尝试endTaskTax", strconv.Itoa(i)))
			res, err := lxhttp.HttpClientPost(lxhttp.NewHttpClient(), clientURL, "application/json", bytes.NewReader(bys))
			if err != nil || strings.Contains(string(res), "Sorry") {
				logger.Error("任务结束请求错误:" + err.Error())
				time.Sleep(time.Second * 10)
				continue
			} else {
				logger.Info(time.Now(), fmt.Sprintf("-----------第%s次endTaskTax成功", strconv.Itoa(i)))
				logger.Info("任务结束发送到服务器返回结果:", string(res))
				break
			}
		}

	}
}

// =================================== 历史数据采集相关 ======================================

// 校验凭证出现的所有科目是否都在中间课余表中
func CheckSubjectCodes(midSubjects []models.HisSubject, Vouchers []models.HisVoucher) bool {
	var Map1 map[string]bool // 凭证的所有科目
	Map1 = make(map[string]bool, len(midSubjects))
	var Map2 map[string]bool // 中间课余表的所有科目
	Map2 = make(map[string]bool, len(midSubjects))
	for _, v1 := range Vouchers {
		for _, v2 := range v1.Items {
			if !Map1[v2.SubjectCode] {
				Map1[v2.SubjectCode] = true
			}
		}
	}
	for _, v := range midSubjects {
		if !Map2[v.SubjectCode] {
			Map2[v.SubjectCode] = true
		}
	}
	for v := range Map1 {
		if !Map2[v] {
			for _, v1 := range Vouchers {
				for _, v2 := range v1.Items {
					if v == v2.SubjectCode {
						logger.Info(v1.VoucherNo, v1.Period)
						break
					}
				}
			}
			logger.Info(v)
			return false
		}
	}
	return true
}

// 返回错误信息
func CheckSubjectCodesError(midSubjects []models.HisSubject, Vouchers []models.HisVoucher) (bool, string) {
	var Map1 map[string]bool // 凭证的所有科目
	Map1 = make(map[string]bool, len(midSubjects))
	var Map2 map[string]bool // 中间课余表的所有科目
	Map2 = make(map[string]bool, len(midSubjects))
	for _, v1 := range Vouchers {
		for _, v2 := range v1.Items {
			if !Map1[v2.SubjectCode] {
				Map1[v2.SubjectCode] = true
			}
		}
	}
	for _, v := range midSubjects {
		if !Map2[v.SubjectCode] {
			Map2[v.SubjectCode] = true
		}
	}
	for v := range Map1 {
		if !Map2[v] {
			msg := ""
			for _, v1 := range Vouchers {
				for _, v2 := range v1.Items {
					if v == v2.SubjectCode {
						msg = fmt.Sprintf("%s期%s凭证出现缺失%s科目", v1.Period, v1.VoucherNo, v)
						logger.Info(v1.VoucherNo, v1.Period)
						break
					}
				}
			}
			logger.Info(v)
			return false, msg
		}
	}
	return true, ""
}
